import org.xiph.system.Bytes;

import flash.Vector;

class System {
    public static inline function
    arraycopy(?src_bytes : Bytes, ?src_int : Array<Int>, src_pos : Int,
              ?dst_bytes : Bytes, ?dst_int : Array<Int>, dst_pos : Int,
              length : Int) : Void {
        if (src_bytes != null) {
            _ac_xiphbytes (src_bytes, src_pos, dst_bytes, dst_pos, length);
        } else if (src_int != null) {
            _ac_arrayint (src_int, src_pos, dst_int, dst_pos, length);
        } else {
            throw "Unimplemented arraycopy for this type!";
        }
    }

    /*
    public static inline function
    _ac_bytes(src : Bytes, src_pos : Int, dst : Bytes, dst_pos : Int,
              length : Int) : Void {
        dst.blit(dst_pos, src, src_pos, length);
    }
    */

    public static inline function
    _ac_xiphbytes(src : Bytes, src_pos : Int, dst : Bytes, dst_pos : Int,
                  length : Int) : Void {
        #if flash9
        src.position = src_pos;
        src.readBytes(dst, dst_pos, length);
        #else
        var buf = src.slice(src_pos, src_pos + length);
        var dsti : Int = dst_pos;
        for (elt in buf) {
            dst[dsti++] = elt;
        }
        #end
    }

    public static inline function
    _ac_arrayint(src : Array<Int>, src_pos : Int,
                 dst : Array<Int>, dst_pos : Int, length : Int) : Void {
        var buf : Array<Int> = src.slice(src_pos, src_pos + length);
        var dsti : Int = dst_pos;
        for (elt in buf) {
            dst[dsti++] = elt;
        }
    }

    public static inline function
    arraycopyV(src : Vector<Int>, src_pos : Int,
               dst : Vector<Int>, dst_pos : Int, length : Int) : Void {
        var buf : Vector<Int> = src.slice(src_pos, src_pos + length);
        var dsti : Int = dst_pos;
        var i : Int = 0;
        var n : Int = buf.length;
        while (i < n) {
            dst[dsti++] = buf[i++];
        }
    }

    public static inline function
    arraycopyVF(src : Vector<Float>, src_pos : Int,
               dst : Vector<Float>, dst_pos : Int, length : Int) : Void {
        var buf : Vector<Float> = src.slice(src_pos, src_pos + length);
        var dsti : Int = dst_pos;
        var i : Int = 0;
        var n : Int = buf.length;
        while (i < n) {
            dst[dsti++] = buf[i++];
        }
    }

    public static inline function fromString(str : String) : Bytes {
        var b = new Bytes();
        b.writeUTFBytes(str);
        b.position = 0;
        return b;
    }

    public static inline function fromBytes(b : Bytes, start : Int, length : Int) : String {
        b.position = start;
        return b.readUTFBytes(length);
    }

    public static function alloc(size : Int) : Bytes {
        var b = new Bytes();
        b.length = size;
        b[size - 1] = 0;
        b.position = 0;
        return b;
    }

    public static inline function abs(n : Int) : Int {
        return n < 0 ? -n : n;
    }

    public static inline function max(a : Int, b : Int) : Int {
        return a > b ? a : b;
    }

    public static function floatToIntBits(value : Float) : Int {
        var b : Bytes = new Bytes();
        b.writeFloat(value);
        b.position = 0;
        var i : Int = b.readInt();
        return i;
    }

    public static function intBitsToFloat(value : Int) : Float {
        var b : Bytes = new Bytes();
        b.writeInt(value);
        b.position = 0;
        var f : Float = b.readFloat();
        return f;
    }
}
