package test;

import haxe.unit.TestCase;

class Dummy {
    public function new() {
    }
}

class TestArrayTools extends TestCase {

    static inline var N = 8;
    static inline var V_INT = 31;
    static inline var V_FLOAT = 3.1415;

    public function test_allocI() {
        var a : Array<Int>;
        var n : Int = N;
        var i : Int = 0;

        a = ArrayTools.allocI(N);
        assertEquals(N, a.length);
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }

        /* unallocated elements ('null') get coerced to Int '0' on reading */
        a = ArrayTools.allocI(N, false);
        assertEquals(N, a.length);
        i = 0;
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }

        a = ArrayTools._allocI(N, true, V_INT);
        i = 0;
        while (i < n) {
            assertEquals(V_INT, a[i]);
            i++;
        }

        a = ArrayTools._allocI(N, false, V_INT);
        i = 0;
        n = N - 1;
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }
        assertEquals(V_INT, a[i]);
    }

    public function test_alloc_Int() {
        var a : Array<Int>;
        var n : Int = N;
        var i : Int = 0;

        a = ArrayTools.alloc(N);
        assertEquals(N, a.length);
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }

        /* unallocated elements ('null') get coerced to Int '0' on reading */
        a = ArrayTools.alloc(N, false);
        assertEquals(N, a.length);
        i = 0;
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }

        a = ArrayTools._alloc(N, true, V_INT);
        i = 0;
        while (i < n) {
            assertEquals(V_INT, a[i]);
            i++;
        }

        a = ArrayTools._alloc(N, false, V_INT);
        i = 0;
        n = N - 1;
        while (i < n) {
            assertEquals(0, a[i]);
            i++;
        }
        assertEquals(V_INT, a[i]);
    }

    public function test_allocF() {
        var a : Array<Float>;
        var n : Int = N;
        var i : Int = 0;

        a = ArrayTools.allocF(N);
        assertEquals(N, a.length);
        while (i < n) {
            assertEquals(0.0, a[i]);
            i++;
        }

        a = ArrayTools._allocF(N, true, V_FLOAT);
        assertEquals(N, a.length);
        i = 0;
        while (i < n) {
            assertEquals(V_FLOAT, a[i]);
            i++;
        }

        /* unallocated elements of Float Arrays are NaN in flash9,
           which can't be represented as a literal in haXe, so don't use it,
           unless you know what you're doing
        */
    }

    public function test_alloc_Float() {
        var a : Array<Float>;
        var n : Int = N;
        var i : Int = 0;

        a = ArrayTools.alloc(N);
        assertEquals(N, a.length);
        while (i < n) {
            assertEquals(0.0, a[i]);
            i++;
        }

        a = ArrayTools._alloc(N, true, V_FLOAT);
        assertEquals(N, a.length);
        i = 0;
        while (i < n) {
            assertEquals(V_FLOAT, a[i]);
            i++;
        }

        /* unallocated elements of Float Arrays are NaN in flash9,
           which can't be represented as a literal in haXe, so don't use it,
           unless you know what you're doing
        */
    }

    public function test_alloc_aClass() {
        var a : Array<Dummy>;
        var n : Int = N;
        var i : Int = 0;
        var v : Dummy = new Dummy();

        a = ArrayTools.alloc(N);
        assertEquals(N, a.length);
        while (i < n) {
            assertEquals(null, a[i]);
            i++;
        }

        /* pretty silly: filling an array with a single instance... */
        a = ArrayTools._alloc(N, true, v);
        assertEquals(N, a.length);
        i = 0;
        while (i < n) {
            assertEquals(v, a[i]);
            i++;
        }
    }

    public function test_copy() {
        var a : Array<Int> = [0, 1, 2, 3, 4, 5, 6, 7];
        var b : Array<Int> = [8, 9, 10, 11, 12, 13, 14, 15];
        var c : Array<Int> = a.copy();

        var t : Array<Int>;
        var v : Array<Int> = [0, 1, 2, 2, 3, 4, 5, 7];

        ArrayTools.arraycopy(a, 2, c, 3, 4);
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        c = a.copy();
        ArrayTools.arraycopy(a, 2, c, 0, 6);
        v = [2, 3, 4, 5, 6, 7, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        c = a.copy();
        ArrayTools.arraycopy(a, 0, c, 2, 6);
        v = [0, 1, 0, 1, 2, 3, 4, 5];
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        c = a.copy();
        ArrayTools.arraycopy(b, 0, c, 0, 8);
        v = [8, 9, 10, 11, 12, 13, 14, 15];
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        c = a.copy();
        ArrayTools.arraycopy(b, 0, c, 0, 5);
        v = [8, 9, 10, 11, 12, 5, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        c = a.copy();
        ArrayTools.arraycopy(c, 2, c, 0, 6);
        v = [2, 3, 4, 5, 6, 7, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], c[i]);
        }

        /* not in-place copy ... */
        c = a.copy();
        t = ArrayTools.copy(a, 2, c, 3, 4);
        assertEquals(8, t.length);
        v = [0, 1, 2, 2, 3, 4, 5, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }

        t = ArrayTools.copy(a, 2, c, 0, 6);
        assertEquals(8, t.length);
        v = [2, 3, 4, 5, 6, 7, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }

        t = ArrayTools.copy(a, 0, c, 2, 6);
        assertEquals(8, t.length);
        v = [0, 1, 0, 1, 2, 3, 4, 5];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }

        t = ArrayTools.copy(b, 0, c, 0, 8);
        assertEquals(8, t.length);
        v = [8, 9, 10, 11, 12, 13, 14, 15];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }

        t = ArrayTools.copy(b, 0, c, 0, 5);
        assertEquals(8, t.length);
        v = [8, 9, 10, 11, 12, 5, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }

        t = ArrayTools.copy(c, 2, c, 0, 6);
        assertEquals(8, t.length);
        v = [2, 3, 4, 5, 6, 7, 6, 7];
        for (i in 0...v.length) {
            assertEquals(v[i], t[i]);
        }
    }
}
